<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WC_Phone_Order_Custom_Shipping_Cost {
	/**
	 * @var float
	 */
	private $cost;

	/**
	 * @var boolean
	 */
	private $enabled;

	/**
	 * WC_Phone_Order_Custom_Shipping_Cost constructor.
	 *
	 * @param $cost float
	 */
	public function __construct( $cost ) {
		$this->cost = (float)$cost;
		$this->disable();
	}

	public function enable() {
		$this->enabled = true;
	}

	public function disable() {
		$this->enabled = false;
	}

	public function is_enabled() {
		return $this->enabled;
	}

	public function apply() {
		// purge packages to trigger recalculate rates
		foreach ( WC()->shipping->get_packages() as $index => $value ) {
			WC()->session->set( 'shipping_for_package_' . $index, '' );
		}

		add_filter( 'woocommerce_package_rates', function ( $rates, $package ) {
			/** @var WC_Shipping_Rate[] $rates */
			$chosen_shipping_method_id = $this->get_chosen_shipping_method_id();
			if ( isset( $rates[ $chosen_shipping_method_id ] ) ) {
				$chosen_shipping_method = $rates[ $chosen_shipping_method_id ];
			} elseif ( count( $rates ) ) {
				$chosen_shipping_method = reset( $rates );
			} else {
				$chosen_shipping_method = null;
			}

			if ( $chosen_shipping_method ) {
				if ( ! $this->is_free_shipping( $chosen_shipping_method ) && ! $this->is_custom_price_shipping_method( $chosen_shipping_method ) ) {
					$rates[ $chosen_shipping_method->get_id() ]->set_cost( $this->cost );
					$rates[ $chosen_shipping_method->get_id() ]->set_taxes( WC_Tax::calc_shipping_tax( $this->cost, WC_Tax::get_shipping_tax_rates() ) );
				} else {
					$this->disable();
				}
			}


			return $rates;
		}, 10, 2 );
	}

	private function get_chosen_shipping_method_id() {
		$chosen_shipping_method  = false;
		$chosen_shipping_methods = WC()->session->get( 'chosen_shipping_methods' );
		if ( $chosen_shipping_methods ) {
			$chosen_shipping_method = reset( $chosen_shipping_methods );
		}

		return $chosen_shipping_method;
	}

	/**
	 * @param $rate WC_Shipping_Rate
	 *
	 * @return bool
	 */
	private function is_custom_price_shipping_method( $rate ) {
		return preg_match( '/^phone_orders_custom_price:\d+$/', $rate->get_id() );
	}

	/**
	 * @param $rate WC_Shipping_Rate
	 *
	 * @return bool
	 */
	private function is_free_shipping( $rate ) {
		return isset( $rate->cost ) ? 0.00 === (float) $rate->cost : false;
	}

	public function get_data() {
		return $this->is_enabled() ? array(
			'cost'    => $this->cost,
			'enabled' => $this->enabled,
		) : array();
	}

	/**
	 * @param $data array|boolean
	 *
	 * @return WC_Phone_Order_Custom_Shipping_Cost
	 */
	public static function make( $data ) {
		if ( ! isset( $data, $data['cost'], $data['enabled'] ) || ! $data ) {
			return new self( null );
		}

		$price = $data['cost'];
		$self  = new self( $price );
		if ( $data['enabled'] ) {
			$self->enable();
		} else {
			$self->disable();
		}

		return $self;
	}
}